//  == インクルード 必要ファイル読み込み ==
#include <Wire.h>
#include "Adafruit_PN532.h"
#include <FtduinoSimple.h>

#define PN532_IRQ   (0)
#define PN532_RESET (0)

Adafruit_PN532 nfc(PN532_IRQ, PN532_RESET);

void setup(void) {
  Serial.begin(115200);

  // USBを最大3秒待つ
  uint32_t to = millis();
  while ((!Serial) && ((millis() - to) < 3000))
    delay(10);

  nfc.begin();

  //  ファームウェアのバージョン情報を格納する変数
  uint32_t versiondata = nfc.getFirmwareVersion();
  // NFCリーダー（PN532）が見つからないとき
  if (! versiondata) {
    Serial.print("PN532がありません");
    while (1); //  停止
  }

  // 見つかった場合
  //  ファームウェアのバージョン情報を表示
  Serial.print("PN532が見つかりました"); Serial.println((versiondata >> 24) & 0xFF, HEX);
  Serial.print("ファームウェアのバージョン : "); Serial.print((versiondata >> 16) & 0xFF, DEC);
  Serial.print('.'); Serial.println((versiondata >> 8) & 0xFF, DEC);

  // RFIDタグを読みとるように構成
  nfc.SAMConfig();
  //  シリアルモニタへ表示
  Serial.println("待機中 ...");
  // 内部LED　ON
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);
}

void loop(void) {
  //  UID Value を格納する配列
  static uint8_t registered_uid[4];
  //  カードの入力がまだないかの状態を表す変数
  static bool registered = false;
  //  返された UID を格納するためのバッファ
  uint8_t uid[] = { 0, 0, 0, 0, 0, 0, 0 };
  //  UIDの長さ （byte数） （ISO14443Aカードタイプに応じて4または7バイト）
  uint8_t uidLength;

  //  カードの入力を待つ
  //  'uid'には UID Value 'uidLength'には UID Length が入る
  if (nfc.readPassiveTargetID(PN532_MIFARE_ISO14443A, uid, &uidLength, 100)) {
    //  カード状態を表示
    Serial.println("ISO14443Aカードが見つかりました");
    Serial.print("  UID Length: "); Serial.print(uidLength, DEC); Serial.println(" bytes");
    Serial.print("  UID Value: ");  nfc.PrintHex(uid, uidLength);
    Serial.println("");

    if (uidLength == 4) {
      Serial.println("-- Mifare Classicカードです --");

      // 最初のカードなら受け入れる
      if (!registered) {
        Serial.println("カードを登録");
        memcpy(registered_uid, uid, uidLength);
        digitalWrite(LED_BUILTIN, LOW); // 内部LED 消灯
        registered = true; // 状態を変更
      } else {
        if (memcmp(registered_uid, uid, uidLength) == 0) {
          Serial.println("== 受け付けました ==");
          // O1のLEDを点灯
          ftduino.output_set(Ftduino::O1, Ftduino::HI);
          ftduino.output_set(Ftduino::O2, Ftduino::LO);
        } else {
          // O2のLEDを点灯
          Serial.println("== カードが拒否されました ==");
          ftduino.output_set(Ftduino::O1, Ftduino::LO);
          ftduino.output_set(Ftduino::O2, Ftduino::HI);
        }
      }
    }
  } else {
    //  カードがない時は　どちらのLEDもOFF
    ftduino.output_set(Ftduino::O1, Ftduino::LO);
    ftduino.output_set(Ftduino::O2, Ftduino::LO);
  }
}
